#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

#include <mpi.h>

#include "mpi-utils.h"

#define TAG_SUM 42

static unsigned int seed (void) {
  struct timeval now;

  (void) gettimeofday(&now, NULL);

  return now.tv_sec ^ now.tv_usec;
}

int main (int argc, char * argv[]) {
  int i;

  int n;
  int local_n;

  int sum;

  int mpi_rank;
  int mpi_size;

  MPI_Init(&argc, &argv);

  MPI_Comm_rank(MPI_COMM_WORLD, &mpi_rank);
  MPI_Comm_size(MPI_COMM_WORLD, &mpi_size);

  if (PROCESS_IS_ROOT(mpi_rank)) {
    if (argc < 2) {
      fprintf(stderr, "usage: %s n\n", argv[0]);
      MPI_Abort(MPI_COMM_WORLD, EXIT_FAILURE);
    }

    n = atoi(argv[1]);
  }

  MPI_Bcast(&n, 1, MPI_INT, ROOT_PROCESS, MPI_COMM_WORLD);
  local_n = BLOCK_SIZE(mpi_rank, mpi_size, n);

  srand(seed());

  sum = 0;
  for (i = 0; i < local_n; i++)
    sum += rand() % 10;

  if (PROCESS_IS_ROOT(mpi_rank)) {
    for (i = 1; i < mpi_size; i++) {
      int part;

      MPI_Recv(&part, 1, MPI_INT, i, TAG_SUM,
	       MPI_COMM_WORLD, MPI_STATUS_IGNORE);

      sum += part;
    }
  } else {
    MPI_Send(&sum, 1, MPI_INT, ROOT_PROCESS, TAG_SUM,
	     MPI_COMM_WORLD);
  }

  root_printf(mpi_rank, "sum: %d\n", sum);

  FINALIZE_AND_SUCCEED;
}
