#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

#include <mpi.h>

#include "mpi-utils.h"

static unsigned int seed (void) {
  struct timeval now;

  (void) gettimeofday(&now, NULL);

  return now.tv_sec ^ now.tv_usec;
}

int main (int argc, char * argv[]) {
  int i;

  int n;
  int local_n;

  int sum;

  int * partials = NULL;

  int mpi_rank;
  int mpi_size;

  MPI_Init(&argc, &argv);

  MPI_Comm_rank(MPI_COMM_WORLD, &mpi_rank);
  MPI_Comm_size(MPI_COMM_WORLD, &mpi_size);

  if (PROCESS_IS_ROOT(mpi_rank)) {
    if (argc < 2) {
      fprintf(stderr, "usage: %s n\n", argv[0]);
      MPI_Abort(MPI_COMM_WORLD, EXIT_FAILURE);
    }

    n = atoi(argv[1]);
  }

  MPI_Bcast(&n, 1, MPI_INT, ROOT_PROCESS, MPI_COMM_WORLD);
  local_n = BLOCK_SIZE(mpi_rank, mpi_size, n);

  srand(seed());

  sum = 0;
  for (i = 0; i < local_n; i++)
    sum += rand() % 10;

  if (PROCESS_IS_ROOT(mpi_rank)) {
    partials = malloc(mpi_size * sizeof(int));

    if (partials == NULL) {
      perror(argv[0]);
      MPI_Abort(MPI_COMM_WORLD, EXIT_FAILURE);
    }
  }

  MPI_Gather(&sum, 1, MPI_INT, partials, 1, MPI_INT,
	     ROOT_PROCESS, MPI_COMM_WORLD);

  if (PROCESS_IS_ROOT(mpi_rank)) {
    sum = 0;

    for (i = 0; i < mpi_size; i++)
      sum += partials[i];

    printf("sum: %d\n", sum);
  }

  free(partials);

  FINALIZE_AND_SUCCEED;
}
